/*
	File:		LibraryManagerUtilities.h

	Contains:	Interface file for ASLM utilities

	Copyright:	 1991-1995 by Apple Computer, Inc., all rights reserved.

*/


#ifndef __LIBRARYMANAGERUTILITIES__
#define __LIBRARYMANAGERUTILITIES__

#ifndef __LIBRARYMANAGER__
#include <LibraryManager.h>
#endif

/*******************************************************************************
** Forward class declarations
********************************************************************************/

#ifdef __cplusplus
	class	TNotifier;
	class	TIterator;
	class	TLibraryFile;
	class	TClassInfo;
	typedef TClassInfo	TFunctionSetInfo;
#else
	typedef	void	TNotifier;
	typedef	void	TIterator;
	typedef void	TLibraryFile;
	typedef void	TFunctionSetInfo;
#endif


/*******************************************************************************
** Routines for loading and unloading the ASLM.
**
** UnloadLibraryManager and LoadLibraryManager can be bad for your health! 
** They should only be used for testing purposes.
********************************************************************************/

#ifdef __cplusplus
	extern "C" {
#endif

Boolean	IsLibraryManagerLoaded(void);
Boolean	LoadLibraryManager(void);	/* returns true if success or already loaded */
void	UnloadLibraryManager(void);

/*******************************************************************************
** C Routines for allocating memory out of TMemoryPools.
**
** The following routines allow C users to allocate memory from TMemoryPools.
** SLMNewOperator is used for doing allocations and SLMDeleteOperator is 
** used for freeing memory. SLMNewOperator will allocate memory out of
** the specified pool. If you pass in NULL for the pool then it will use
** default memory allocation. The default memory allocation is to allocate
** out of the the library's local pool if the library was built with
** memory=local, and out of the client's local pool if the library was built
** with memory=client. If you don't want to use default memory allocation then
** you can call GetLocalPool to get the library's local pool, and GetClientPool
** to get the current client's local pool.
********************************************************************************/

void*	SLMNewOperator(size_t, TMemoryPool*);
void	SLMDeleteOperator(void*);

/*******************************************************************************
** GetSLMVersion
**
** Returns the version of the installed ASLM in the 'vers'
** resource format (the first 4 bytes only). Returns 0 if the
** ASLM is not installed.
********************************************************************************/

unsigned long GetSLMVersion(void);

/*******************************************************************************
** RegisterInspector
**
** Should only be used by Inspector application.
********************************************************************************/

void RegisterInspector(void*);

/*******************************************************************************
** Trace
**
** Used to send output to the Trace Montior's Window.
********************************************************************************/

void Trace(const char *formatStr, ...);

/*******************************************************************************
** CallInitLibraryManager
**
** CallInitLibraryManager is called automatically when an application first
** starts up. It is called before any static objects are constructed. It's
** purpose is to allow applications to call InitLibraryManager before static
** objects are constructed so they can be instances of shared classes and can also
** allocate memory using the ASLM global new operator. A default implementation
** is provided in LibraryManager.o that does nothing at all. If you want it to
** actually call InitLibraryManager, you must provide your own implementation
** that makes the InitLibraryManager call and returns the result. Make sure you
** link it in before LibraryManager.o. If anything besides kNoError is returned
** then the static objects will not be initalized. If you use this method of
** calling InitLibraryManager, then do no call CleanupLibraryManager or a crash
** will occur when the static objects are destructed. ASLM will call it
** automatically when the application quits after static object destructors
** are called.
********************************************************************************/

OSErr CallInitLibraryManager();

/*******************************************************************************
** SLMsprintf
**
** The SLMsprintf routine in stdclib.o won't work with ASLM libraries.
** Use the one in LibraryManager.o instead.
********************************************************************************/

int SLMsprintf(char *outString, const char *argp, ...);				
int SLMvsprintf(char* outstring, const char* format, char* args);

/*******************************************************************************
** Utility functions
**
** %%% These functions need to be modified if unsigned short is not 16 bits
********************************************************************************/

#ifdef __cplusplus

inline unsigned short HighWord(unsigned long l)
{
	return ((unsigned short)(l >> 16));
}

inline unsigned short LowWord(unsigned long l)
{
	return (unsigned short)l;
}

inline unsigned char HighByte(unsigned short l)
{
	return (unsigned char)(l >> 8);
}

inline unsigned char LowByte(unsigned short l)
{
	return (unsigned char)l;
}

#else

#define HighWord(x)		((unsigned short)((x) >> 16))
#define LowWord(x)		(((unsigned short)(x)))
#define HighByte(x)		((unsigned char)((x) >> 8))
#define LowByte(x)		(((unsigned char)(x)))

#endif

/*******************************************************************************
** Atomic Bit functions
**
** These functions atomically set, clear, and test bits. SetBit and ClearBit
** return the previous value of the bit. AtomicSetBoolean and AtomicClearBoolean
** return true if the bit was set or cleared by the caller and false if the
** bit was already set or cleared before the caller tried to set or clear it.
********************************************************************************/

Boolean	SetBit(void* mem, size_t bitno);
Boolean	ClearBit(void* mem, size_t bitno);
Boolean	TestBit(const void* mem, size_t bitno);

#if USES68KINLINES

	#pragma parameter __D0 AtomicSetBoolean(__A0)
	Boolean AtomicSetBoolean(unsigned char*) =
	{
		0x08d0, 0x00, 0x57c0, 0x0200, 0x0001	/* bset.b #0,(a0); seq d0, andi.b #$01,d0 */
	};
	
	#pragma parameter __D0 AtomicClearBoolean(__A0)
	Boolean AtomicClearBoolean(unsigned char*) =
	{
		0x0890, 0x00, 0x56c0, 0x0200, 0x0001	/* bclr.b #0,(a0); sne d0, andi.b #$01,d0 */
	};
	
	#pragma parameter __D0 AtomicTestBoolean(__A0)
	Boolean AtomicTestBoolean(unsigned char*) =
	{
		0x0810, 0x00, 0x56c0, 0x0200, 0x0001	/* btst.b #0,(a0); sne d0, andi.b #$01,d0 */
	};

#else
	#ifdef __cplusplus

		inline Boolean AtomicSetBoolean(unsigned char* theBoolean)
		{
			return !SetBit(theBoolean, 7);
		}
		inline Boolean AtomicClearBoolean(unsigned char* theBoolean)
		{
			return ClearBit(theBoolean, 7);
		}
		inline Boolean AtomicTestBoolean(unsigned char* theBoolean)
		{
			return TestBit(theBoolean, 7);
		}

	#else

		#define AtomicSetBoolean(theBoolean)	!SetBit(theBoolean, 7)
		#define AtomicClearBoolean(theBoolean)	ClearBit(theBoolean, 7)
		#define AtomicTestBoolean(theBoolean)	TestBit(theBoolean, 7)

	#endif
#endif

/******************************************************************************
** Memory Functions
**
** SLMmemcpy, SLMmemmove, and SLMmemset are equivalent to the C memcpy, memmove,
** and memset routines. They are exported by SLM and are faster then the C versions.
*******************************************************************************/

void	ZeroMem(void* dest, size_t nBytes);
void* 	SLMmemcpy(void* dest, const void* src, size_t nBytes);
void* 	SLMmemmove(void* dest, const void* src, size_t nBytes);
void*	SLMmemset (void *dest, int c, size_t n);

/**********************************************************************
** struct TFileSpec
**
** TFileSpec is a struct for specifying the location of a library
** file (TLibraryFile) in a file system or OS independent way. The 
** "subclasses" contain the details and the "base class" is used to compare
** them or to pass them around without worry about the contents.
**
** Currently only the TMacFileSpec is supported since this is how the
** SLM tracks library files on the Mac OS. There is a chance that in the
** furture it may track files under System 7.0 using TFileIDFileSpec so
** you should write your 7.0 code to handle either type. The
** IsFileSpecTypeSupported() routine will tell you if the specified
** TFileSpec subclass is supported.
**
** Generally you don't need to be concerned with with TFileSpecs unless
** you are going to call RegisterLibraryFile(), RegisterLibraryFileFolder(),
** or GetFileSpec().
***********************************************************************/

/* Different types of TFileSpecs we can have. */

#ifndef __cplusplus

typedef unsigned int	FileSpecType;

#define kUnknownType	((FileSpecType)0)
#define kMacType		((FileSpecType)1)
#define kFileIDType		((FileSpecType)2)
#define kImageType		((FileSpecType)3)
#define kMacRomType		((FileSpecType)9)
#define kMaxType		((FileSpecType)255)

Boolean IsFileSpecTypeSupported(FileSpecType);
Boolean CompareFileSpecs(const void* f1, const void* f2);

struct TFileSpec
{
	unsigned char	fType;		/* FileSpectype */
	unsigned char	fSize;		/* size of struct */
};
typedef struct TFileSpec TFileSpec;

/**********************************************************************
** struct TMacFileSpec
**
** TMacFileSpec keeps track of the file by using an file name, volume
** refNum, and directory id. You must use InitMacFileSpec to make
** sure that the length is set properly.
***********************************************************************/

struct TMacFileSpec
{
	unsigned char	fType;		/* FileSpec type						*/
	unsigned char	fSize;		/* size of struct						*/
	short			fVRefNum;	/* volume refNum of volume file is on	*/
	long			fParID;		/* dirID of the folder file is in		*/
	Str63			fName;		/* name of the file						*/
};		
typedef struct TMacFileSpec TMacFileSpec;
	
void InitMacFileSpec(TMacFileSpec *spec, int vRefNum, long parID, Str63 name);

/**********************************************************************
** struct TFileIDFileSpec
**
** TFileIDFileSpec keeps track of library files by fileID and vRefNum.
** You must use InitFileIDFileSpec to make sure that the length is set
** properly
***********************************************************************/

/* Some macros to make accessing fields without doing a cast easier */
#define GetFileIDFromFileSpec(x)	(((const TFileIDFileSpec*)x)->fFileID)
#define GetVRefNumFromFileSpec(x)	(((const TFileIDFileSpec*)x)->fVRefNum)

struct TFileIDFileSpec
{
	unsigned char	fType;		/* FileSpec type	*/
	unsigned char	fSize;		/* size of struct	*/
	short			fVRefNum;	/* volume refNum	*/
	long			fFileID;	/* FileID			*/
};
typedef struct TFileIDFileSpec TFileIDFileSpec;

void InitFileIDFileSpec(TFileIDFileSpec *spec, int vRefNum, long fileID);

#endif

/*******************************************************************************
** RegisterLibraryFile/UnregisterLibraryFile
**
** Used to register or unregister a library file. Currently only the
** TMacFileSpec is supported. You'll get a kASLMNotSupportedErr error for anything else.
**
** RegisterLibraryFile returns kNoError if successful or kASLMFileNotFoundErr if
** an error ocurred while trying to access the file. Other error codes are also
** possible if there was trouble processing the file such as kASLMOutOfMemoryErr. If
** created the TLibraryFile is returned in the TLibraryFile** parameter. You
** can pass in null if you don't want the value returned.
**
** UnregisterLibraryFile accepts a "forceUnload" parameter. If true is passed
** then it will force all loaded libraries in the library file to be
** unloaded even if they are in use. Unless you are sure that loaded libraries
** can be safely unloaded you should pass false. In this case the library file
** will not be deleted until all of its libraries are unloaded.
** kNoError will be returned if successful and wil be returend kASLMNotSupportedErr if
** the folder was never registered.
********************************************************************************/

OSErr	RegisterLibraryFile(const TFileSpec*, TLibraryFile**);
OSErr	UnregisterLibraryFile(TLibraryFile*, BooleanParm forceUnload);
OSErr	UnregisterLibraryFileByFileSpec(const TFileSpec*, BooleanParm forceUnload);

/*******************************************************************************
** RegisterLibraryFileFolder/UnregisterLibraryFileFolder
**
** Used to register a folder that contains library files in it. The SLM will keep
** track of library files dragged into and out of this folder. Currently only the
** TMacFileSpec is supported. You'll get a kASLMNotSupportedErr error for anything else.
**
** RegisterLibraryFileFolder returns kNoError if successful or kASLMFileNotFoundErr if
** an error ocurred while trying to access the folder.
**
** UnregisterLibraryFileFolder accepts a "forceUnload" parameter. If true is passed
** then it will unregister the folder and force all loaded libraries to be
** unloaded even if they are in use. Unless you are sure that loaded libraries
** can be safely unloaded you should pass false. kNoError will be returned if
** successful, kASLMFolderNotFoundErr if the folder was never registered, and
** kASLMFolderInUseErr if any libraries in the folder were loaded and false was passed
** in the forceUnload parameter.
********************************************************************************/

OSErr	RegisterLibraryFileFolder(const TFileSpec*);
OSErr	UnregisterLibraryFileFolder(const TFileSpec*, BooleanParm forceUnload);

/*******************************************************************************
** UnloadUnusedLibraries
**
** UnloadUnusedLibraries forces any libraries the are currently scheduled to
** be unloaded (because they are no longer being used) to be unloaded immediately
** rather then letting ASLM wait up to 1 second before unloading them. This
** routine is rarely needed, but may be necessary if you have complex library
** dependencies and are having trouble getting your libraries to unload.
********************************************************************************/

void	UnloadUnusedLibraries(void);

/*******************************************************************************
** EnterSystemMode/LeaveSystemMode
**
** These functions should bracket calls that open files or get memory that needs
** to hang around after an application quits
********************************************************************************/

void*	EnterSystemMode(void);
void	LeaveSystemMode(void*);

/*******************************************************************************
** Death Notification
********************************************************************************/

Boolean InstallDeathWatcher(TNotifier* notifier);
Boolean RemoveDeathWatcher(TNotifier* notifier);

/*******************************************************************************
** EnterInterrupt/LeaveInterrupt
**
** These functions should be called when you are in an interrupt service routine
** or a deferred task and you want to do something that will cause ASLM
** code to be executed such as allocating pool memory or newing an object. The
** ASLM needs to know that it is at interrupt time so it doesn't do
** anything stupid like try to allocate memory or load library code. This doesn't
** mean that all ASLM calls are safe at interrupt time, just that the
** ones that claim to be safe will only be safe if you do an EnterInterrupt call
** first.
**
** You don't need to use these routines when your interrupt service routine is
** scheduling an operation on a TInterruptScheduler, when the operation gets
** executed at deferred task time, or when a TTimeScheduler operation gets
** executed. In the former case the ASLM is smart enough to realize
** that you are at interrupt time and in the later two cases the
** ASLM does an EnterInterrupt before calling your operation and a
** LeaveInterrupt when your operation returns.
********************************************************************************/

void	EnterInterrupt(void);
void	LeaveInterrupt(void);

/*******************************************************************************
** AtInterruptLevel
**
** This function returns true if we are currently executing at non-system-task
** time.
********************************************************************************/

Boolean AtInterruptLevel(void);

/*******************************************************************************
** InInterruptScheduler
**
** This function returns true if we are currently running an interrupt scheduler
********************************************************************************/

Boolean InInterruptScheduler(void);

/**********************************************************************
** GlobalWorld routines
**
** InitGlobalWorld will create and initialize the global world for
** standalone code on the MacOS such as INITs and CDEVs. It also does
** a SetCurrentGlobalWorld. FreeGlobalWorld will free the memory used
** by the global world created by InitGlobalWorld.
**
** EnterCodeResource is also used by stand alone code resources. It is
** most useful when the code resource only calls InitLibraryManager once
** but may then be reentered multiple times before calling CleanupLibraryManager.
** EnterCodeResoruce will set the current global world to the code resources
** global world and set the code resource as the current client.
** LeaveCodeResource will undo what EnterCodeResource does. These routines
** are NOT reentrant. You must call InitCodeResource before calling
** EnterCodeResource. It will call InitGlobalWorld and save the global world
** pointer in a PC-relative location so EnterCodeResource can access it.
**
** SetCurrentGlobalWorld and GetCurrentGlobalWorld used for getting and
** setting A5 on the MacOS.
**
** GetGlobalWorld is used to get the global world pointer for a Library.
** OpenGlobalWorld will make the library's global world the current global
** world. It's the same as calling SetCurrentGlobalWorld(GetGlobalWorld()).
** CloseGlobalWorld is used to revert back to the global world that was
** current before calling OpenGlobalWorld. It's the same as calling
** SetCurrentGlobalWorld(oldWorld) except that it doesn't return a
** global world.
**
** Since libraries are always compiled with model far, it's
** not necessary to do an OpenGlobalWorld before accessing globals or
** making intersegment calls. Their only purpose is to make the library
** the current "Client" since the client is determined by the current
** value of A5 on the MacOS.
**
** ONLY LIBRARIES AND MODEL FAR CLIENTS SHOULD CALL Get/Open/CloseGlobalWorld.
********************************************************************************/

GlobalWorld ASLMGetCurrentGlobalWorld(void);
GlobalWorld ASLMSetCurrentGlobalWorld(GlobalWorld newWorld);

#if USES68KINLINES

	#pragma parameter __D0 SetCurrentGlobalWorld(__A0)
	GlobalWorld SetCurrentGlobalWorld(GlobalWorld newWorld)
		= {0x200D,						/* move.l	A5,D0		*/
		   0x2A48};						/* move.l	A0,A5		*/
		   
	GlobalWorld GetCurrentGlobalWorld()
		= {0x200D};						/* move.l	A5,D0		*/

#else

	#define GetCurrentGlobalWorld ASLMGetCurrentGlobalWorld
	#define SetCurrentGlobalWorld ASLMSetCurrentGlobalWorld

#endif


OSErr				InitGlobalWorld(void);		/* called by standalone code only!!! */
void				FreeGlobalWorld(void);		/* called by standalone code only!!! */

OSErr				InitCodeResource(void);		/* called by standalone code only!!! */
void				EnterCodeResource(void);	/* called by standalone code only!!! */
void				LeaveCodeResource(void);	/* called by standalone code only!!! */
	
TLibraryManager*	GetCurrentClient(void);
TLibraryManager*	GetSystemClient(void);		/* returns the ASLM client */
TLibraryManager*	GetLibraryClient(TLibrary*);/* returns the library's client */

TLibraryManager*	SetCurrentClient(TLibraryManager*);
TLibraryManager*	SetSelfAsClient(void);		/* set owner of code making call as current client */
TLibraryManager*	SetSystemAsClient(void);	/* set ASLM itself as current client */
#if GENERATING68K
TLibraryManager*	SetClientToWorld(void);		/* set owner of current global world as current client */
TLibraryManager*	GetClientFromWorld(GlobalWorld);	/* get client which owns the specified global world */
#endif
	
GlobalWorld 	GetGlobalWorld(void);
GlobalWorld		OpenGlobalWorld(void);
void			CloseGlobalWorld(GlobalWorld oldWorld);

/*	-------------------------------------------------------------------------
	Inline implementation for global world functions
	------------------------------------------------------------------------- */

#ifdef __cplusplus
	inline GlobalWorld GetGlobalWorld()
	{
		return GetLocalLibraryManager()->GetGlobalWorld();
	}
	inline GlobalWorld OpenGlobalWorld()
	{
		return SetCurrentGlobalWorld(GetGlobalWorld());
	}
	inline void	CloseGlobalWorld(GlobalWorld oldWorld)
	{
		SetCurrentGlobalWorld(oldWorld);
	}
#else
	#define GetGlobalWorld()		((GlobalWorld)(((void**)GetLocalLibraryManager())[4]))
	#define OpenGlobalWorld()		SetCurrentGlobalWorld(GetGlobalWorld())
	#define CloseGlobalWorld(world)	SetCurrentGlobalWorld(world)
#endif

/**********************************************************************
** TLibraryFile "C" interface
**
** The C interface to the TLibraryFile class defined in 
** LibraryManagerClasses.h. Used mainly to get resources out of a library.
********************************************************************************/

TLibraryFile*	GetLocalLibraryFile();

Ptr			GetSharedResource(TLibraryFile*, ResType, int theID, OSErr*);
Ptr			GetSharedIndResource(TLibraryFile*, ResType, int index, OSErr*);
Ptr			GetSharedNamedResource(TLibraryFile*, ResType,
								   const char* name, OSErr*);

void		ReleaseSharedResource(TLibraryFile*, Ptr);
long		CountSharedResources(TLibraryFile*, ResType);

size_t		GetSharedResourceUseCount(TLibraryFile*, Ptr);
OSErr		GetSharedResourceInfo(TLibraryFile*, Ptr, size_t* theSize,
								  short* theID, ResType*, char* theName);

TFileSpec*	GetFileSpec(TLibraryFile*);
long		GetRefNum(TLibraryFile*);

OSErr		OpenLibraryFile(TLibraryFile*);
OSErr		CloseLibraryFile(TLibraryFile*);
		
OSErr		Preflight(TLibraryFile*, long* savedRefNum);
OSErr		Postflight(TLibraryFile*, long savedRefNum);

/**********************************************************************
** Miscellaneous TLibrary routines.
***********************************************************************/

TLibraryFile*	GetLibraryFile(TLibrary*);

#ifdef __cplusplus
	TLibraryID* GetLibraryID(TLibrary*);
#else
	TLibraryID GetLibraryID(TLibrary*);
#endif

/**********************************************************************
** Routines for loading and unloading library code segments.
***********************************************************************/

#if GENERATING68K
OSErr	LoadCodeSegmentByNumber(TLibrary*, int segmentNumber);
OSErr	LoadCodeSegmentByName(TLibrary*, CDECLProcPtr theRoutine);
OSErr	UnloadCodeSegmentByNumber(TLibrary*, int segmentNumber);
OSErr	UnloadCodeSegmentByName(TLibrary*, CDECLProcPtr theRoutine);
#endif

/*********************************************************************
** Routines that will get a TLibrary object
**********************************************************************/

#ifdef __cplusplus

TLibrary*	GetLocalLibrary();
TLibrary*	LookupLibrary(const TLibraryID&);
TLibrary*	LookupLibraryWithClassID(const TClassID&);
TLibrary*	LookupLibraryWithFunctionSetID(const TFunctionSetID&);

#else
	
TLibrary*	GetLocalLibrary();
TLibrary*	LookupLibrary(const TLibraryID);
TLibrary*	LookupLibraryWithClassID(const TClassID);
TLibrary*	LookupLibraryWithFunctionSetID(const TFunctionSetID);

#endif

/*********************************************************************
** Routines for getting all libraries and library files.
**
** CreateLibraryIterator returns an iterator that will iterate over all
** publicly known libraries and CreateLibraryFileIterator will return
** an iterator that will iterate over all publicly known library files.
** Make sure you delete the iterator when you are done.
**********************************************************************/

TIterator* CreateLibraryIterator(TLibraryManager* mgr);
TIterator* CreateLibraryFileIterator(TLibraryManager* mgr);

/*********************************************************************
** Per Client data routines
**********************************************************************/

void*	GetClientData(void);
void*	GetLibraryClientData(TLibrary*);

/**********************************************************************
** TClassInfo "C" interface
**
** The C interface to the TClassInfo class. Used mainly to get information
** about function sets. It is most useful for iterating over all function sets
** with a common interface. This can be done by giving the functions sets the
** same "parent id" when exporting each function set.
**
** Note, TClassInfo can also be used to iterate over function sets and
** the routines given below can also be used iterate over classes.
***********************************************************************/

#ifdef __cplusplus

TFunctionSetInfo*	GetFunctionSetInfo(const TFunctionSetID&, OSErr* = NULL);
void				FreeFunctionSetInfo(TFunctionSetInfo*);

void				FSInfoReset(TFunctionSetInfo*);		// start iteration over
TFunctionSetID*		FSInfoNext(TFunctionSetInfo*);		// get next function set
Boolean				FSInfoIterationComplete(TFunctionSetInfo*);	// true if we are done iterating

TFunctionSetID*		FSInfoGetFunctionSetID(TFunctionSetInfo*);
TFunctionSetID*		FSInfoGetInterfaceID(TFunctionSetInfo*, size_t idx = 0);
TLibrary*			FSInfoGetLibrary(TFunctionSetInfo*);
TLibraryFile*		FSInfoGetLibraryFile(TFunctionSetInfo*);
unsigned short		FSInfoGetVersion(TFunctionSetInfo*);
unsigned short		FSInfoGetMinVersion(TFunctionSetInfo*);

#else

TFunctionSetInfo*	GetFunctionSetInfo(TFunctionSetID, OSErr*);
void				FreeFunctionSetInfo(TFunctionSetInfo*);

void				FSInfoReset(TFunctionSetInfo*);		/* start iteration over */
TFunctionSetID		FSInfoNext(TFunctionSetInfo*);		/* get next function set */
Boolean				FSInfoIterationComplete(TFunctionSetInfo*);	/* true if we are done iterating */

TFunctionSetID		FSInfoGetFunctionSetID(TFunctionSetInfo*);
TFunctionSetID		FSInfoGetInterfaceID(TFunctionSetInfo*, size_t idx);
TLibrary*			FSInfoGetLibrary(TFunctionSetInfo*);
TLibraryFile*		FSInfoGetLibraryFile(TFunctionSetInfo*);
unsigned short		FSInfoGetVersion(TFunctionSetInfo*);
unsigned short		FSInfoGetMinVersion(TFunctionSetInfo*);

#endif

/*******************************************************************************
** Debugging Macros and inlines
********************************************************************************/

void DebugBreakProc(BooleanParm, const char*);

#ifdef __cplusplus
}

inline void DoDebugBreak(BooleanParm value, const char* str)
{
	DebugBreakProc(value, str);
}

inline void DoDebugBreak(const char* str)
{
	DebugBreakProc(true, str);
}


#define ForceDebugBreak(str)		DoDebugBreak(true, str)

#if qDebug
	#define DebugBreak(str)			DoDebugBreak(true, str)
	#define DebugTest(val, str)		DoDebugBreak(val, str)
#else
	#define DebugBreak(str)
	#define DebugTest(val, str)
#endif

#else

#if qDebug
	#define DebugBreak(str)			DebugBreakProc(true, str)
	#define DebugTest(val, str)		DebugBreakProc(val, str)
#else
	#define DebugBreak(str)
	#define DebugTest(val, str)
#endif

#endif

/*******************************************************************************
** TAtomicBoolean
**
** Set returns true if you were the "setter".
** Clear returns true if you were the "clearer".
** Test returns the current state of the boolean.
********************************************************************************/

#ifdef __cplusplus

struct TAtomicBoolean
{
		void		Init();
		Boolean		Set();
		Boolean		Clear();
		Boolean		Test();
		
	unsigned char	fFlag;
};

	inline void TAtomicBoolean::Init()
	{
		fFlag = 0;
	}

	inline Boolean TAtomicBoolean::Set()
	{
		return AtomicSetBoolean(&fFlag);
	}

	inline Boolean TAtomicBoolean::Clear()
	{
		return AtomicClearBoolean(&fFlag);
	}
	
	inline Boolean TAtomicBoolean::Test()
	{
		return AtomicTestBoolean(&fFlag);
	}

#endif	

/*******************************************************************************
** TUseCount
********************************************************************************/

#ifdef __cplusplus

#if USES68KINLINES

	#pragma parameter __D0 IncrementUseCount(__A0)
	extern "C" Boolean IncrementUseCount(long*) =
	{
		0x5290,		/*	addq.l	#1,(a0)	*/
		0x57c0		/*  seq		d0		*/
	};
	
	#pragma parameter __D0 DecrementUseCount(__A0)
	extern "C" Boolean DecrementUseCount(long*) =
	{
		0x5390,		/*	subq.l	#1,(a0)	*/
		0x5bc0		/*  smi		d0		*/
	};

#else

	extern "C" Boolean IncrementUseCount(long*);
	extern "C" Boolean DecrementUseCount(long*);

#endif

struct TUseCount
{
		void		SetValue(long);
		void		SetUseCount(long);
		long		GetValue() const;
		long		GetUseCount() const;
		void		Init();
		Boolean		Increment();		// Returns True if first time
		Boolean		Decrement();		// Returns True if back to unused
		Boolean		IsUnused() const;
		
	long	fValue;
};

/*	-------------------------------------------------------------------------
	Inline methods for TUseCount
	------------------------------------------------------------------------- */

	inline void TUseCount::SetValue(long val)
	{
		fValue = val;
	}
	
	inline void TUseCount::SetUseCount(long val)
	{
		fValue = val - 1;
	}
	
	inline void TUseCount::Init()
	{
		fValue = -1;
	}
	
	inline Boolean TUseCount::IsUnused() const
	{
		return fValue < 0;
	}
	
	inline long TUseCount::GetValue() const
	{
		return fValue;
	}
	
	inline long TUseCount::GetUseCount() const
	{
		return fValue + 1;
	}
	
	inline Boolean TUseCount::Increment()
	{
		return IncrementUseCount(&fValue);
	}
	
	inline Boolean TUseCount::Decrement()
	{
		return DecrementUseCount(&fValue);
	}
	
#endif	

#endif
